@@ -2,36 +2,37 @@
 //! This library provides a number of ways to compute the bit reversal of all primitive integers.
 //! There are currently 3 different algorithms implemented: Bitwise, Parallel, and Lookup reversal.
 //!
-//! ## YMMV Performance Comparison
-//! Relatively bitwise is slower than parallel and lookup reversal. Parallel seems to be slightly
-//! slower than lookup for small sized integers, but regaining a lead for larger sized integers.
-//!
-//! ## Memory Consumption
-//! Bitwise uses the least amount of memory only using three integers to compute the reversal.
-//! Parallel allocates 3 constants of the same size of the type being reversed.
-//! Lookup allocates 256 u8s or 256 bytes to do its byte lookup reversal.
-//!
 //! ## Example
 //! ```
 //! use bit_reverse::ParallelReverse;
 //!
 //! assert_eq!(0xA0u8.swap_bits(), 0x05u8);
 //! ```
-
+//! This library is very simple uses just import the crate and the algorithm you want to use. Then
+//! you can call swap_bits() on any primitive integer. If you want to try a different algorithm
+//! just change the use statement and now your program will use the algorithm instead.
+//!
+//! ## YMMV Performance Comparison
+//! I wouldn't use BitwiseReverse as it is mainly there for completeness and is strictly inferior
+//! to ParallelReverse, which is a Bitwise Parallel Reverse and thus an order of magnitude faseter.
+//! I would recommend you use the ParallelReverse as it performs equal to or better than
+//! LookupReverse for all types. Plus as an added bonus doesn't eat your cache with a lookup table.
+//!
+//! ## Memory Consumption
+//! Bitwise uses the least amount of memory only using three integers to compute the reversal.
+//! Parallel allocates 3 constants of the same size of the type being reversed.
+//! Lookup allocates 256 u8s or 256 bytes to do its byte lookup reversal.
 
 // This library abuse overflowing literals to be able to use macros to reduce duplicate code.
 #![allow(overflowing_literals)]
 
-// #![feature(test)]
-// extern crate test;
-
 /// Computes bit reversal by going bit by bit and setting the reverse position bit for the output.
 pub trait BitwiseReverse<T> {
     /// Swaps the bits such that bit i is now bit N-i, where N is the length of the T in bits.
     fn swap_bits(self) -> T;
 }
 
-macro_rules! doit_bitwise { ($($ty:ty)*) => ($(
+macro_rules! doit_bitwise { ($($ty:ty),*) => ($(
     impl BitwiseReverse<$ty> for $ty {
         // This algorithm uses the reverse variable as a like a stack to reverse the value.
         // The lesser significant bits are pushed onto the reverse variable and then the variable
@@ -62,7 +63,7 @@ macro_rules! doit_bitwise { ($($ty:ty)*) => ($(
     })*)
 }
 
-doit_bitwise!(u8 u16 u32 u64 usize i8 i16 i32 i64 isize);
+doit_bitwise!(u8, u16, u32, u64, usize);
 
 /// Computes bit reversal by using a divide and conquer approach. Pairs of bits are swapped.
 /// Then neighboring bit pairs are swapped. Each time swapping the next largest group of bits.
@@ -72,7 +73,7 @@ pub trait ParallelReverse<T> {
     fn swap_bits(self) -> T;
 }
 
-macro_rules! doit_parallel { ($($ty:ty)*) => ($(
+macro_rules! doit_parallel { ($($ty:ty),*) => ($(
     impl ParallelReverse<$ty> for $ty {
         fn swap_bits(self) -> $ty {
             let mut v = self;
@@ -88,7 +89,7 @@ macro_rules! doit_parallel { ($($ty:ty)*) => ($(
     })*)
 }
 
-doit_parallel!(u8 u16 u32 u64 usize i8 i16 i32 i64 isize);
+doit_parallel!(u8, u16, u32, u64, usize);
 
 /// Computes bit reversal by using lookup table to translate a single byte into its reverse.
 /// For multi-byte types, the byte order is swapped to complete the reversal.
@@ -166,6 +167,42 @@ impl LookupReverse<usize> for usize {
     }
 }
 
+macro_rules! doit_signed {
+    ($($Algo:ident),*) => ($(
+        impl $Algo<i8> for i8 {
+            fn swap_bits(self) -> i8 {
+                $Algo::swap_bits(self as u8) as i8
+            }
+        }
+
+        impl $Algo<i16> for i16 {
+            fn swap_bits(self) -> i16 {
+                $Algo::swap_bits(self as u16) as i16
+            }
+        }
+
+        impl $Algo<i32> for i32 {
+            fn swap_bits(self) -> i32 {
+                $Algo::swap_bits(self as u32) as i32
+            }
+        }
+
+        impl $Algo<i64> for i64 {
+            fn swap_bits(self) -> i64 {
+                $Algo::swap_bits(self as u64) as i64
+            }
+        }
+
+        impl $Algo<isize> for isize {
+            fn swap_bits(self) -> isize {
+                $Algo::swap_bits(self as usize) as isize
+            }
+        }
+    )*)
+}
+
+doit_signed!(BitwiseReverse, ParallelReverse, LookupReverse);
+
 macro_rules! test_suite {
     ($name:ident, $algo:path) => (
         #[cfg(test)]
@@ -196,6 +233,31 @@ macro_rules! test_suite {
             fn reverse_usize() {
                 assert_eq!(0xFFusize.swap_bits(), 0xFFusize.swap_bytes());
             }
+
+            #[test]
+            fn reverse_i8() {
+                assert_eq!(0xABi8.swap_bits(), 0xD5i8);
+            }
+
+            #[test]
+            fn reverse_i16() {
+                assert_eq!(0xABCDi16.swap_bits(), 0xB3D5i16);
+            }
+
+            #[test]
+            fn reverse_i32() {
+                assert_eq!(0xABCD2345i32.swap_bits(), 0xA2C4B3D5i32);
+            }
+
+            #[test]
+            fn reverse_i64() {
+                assert_eq!(0x0123456789ABCDEFi64.swap_bits(), 0xF7B3D591E6A2C480i64);
+            }
+
+            #[test]
+            fn reverse_isize() {
+                assert_eq!(0xFFisize.swap_bits(), 0xFFisize.swap_bytes());
+            }
         }
     )
 }
@@ -203,54 +265,3 @@ macro_rules! test_suite {
 test_suite!(test_bitwise_reverse, BitwiseReverse);
 test_suite!(test_parallel_reverse, ParallelReverse);
 test_suite!(test_lookup_reverse, LookupReverse);
-
-macro_rules! benchmark_suite {
-    ($name:ident, $algo:path) => (
-        #[cfg(test)]
-        mod $name {
-            use super::test;
-            use $algo;
-
-            #[bench]
-            fn reverse_u8(b: &mut test::Bencher) {
-                b.iter(|| {
-                    let n = test::black_box(0xEDu8);
-
-                    n.swap_bits()
-                });
-            }
-
-            #[bench]
-            fn reverse_u16(b: &mut test::Bencher) {
-                b.iter(|| {
-                    let n = test::black_box(0xABCDu16);
-
-                    n.swap_bits()
-                });
-            }
-
-            #[bench]
-            fn reverse_u32(b: &mut test::Bencher) {
-                b.iter(|| {
-                    let n = test::black_box(0xABCD2345u32);
-
-                    n.swap_bits()
-                });
-            }
-
-            #[bench]
-            fn reverse_u64(b: &mut test::Bencher) {
-                b.iter(|| {
-                    let n = test::black_box(0xFEDCBA9876543210u64);
-
-                    n.swap_bits()
-                });
-            }
-        }
-    )
-}
-
-// Uncomment these lines and the test feature and crate at the top of the file to run the benchs.
-// benchmark_suite!(benchmark_parallel_reverse, ParallelReverse);
-// benchmark_suite!(benchmark_lookup_reverse, LookupReverse);
-// benchmark_suite!(benchmark_bitwise_reverse, BitwiseReverse);
